AS::Particle = Struct.new(:pos, :vel, :t)

class AS::ParticleEmitter
	attr_accessor :pos, :scat
	attr_accessor :vel
	attr_accessor :tex
	
	attr_accessor :colora, :colorb
	
	def initialize(burst, rate, life, func)
		@rate = rate.to_i
		@life = life.to_i
		@burst = burst.to_i
		@size = @burst*@life/@rate
		@parts = []
		@ticks = 0
		@partIndex = 0
		
		@func = func
	end
	
	def setVScat(mag, angle)
		@vMagScat = mag
		@vAngleScat = angle*3.141/180.0
	end
	
	def setScale(min, max)
		@smin, @smax = min, max
	end

	def emitParticle()
		th = @vAngleScat*(0.5 - rand)
		ab = 1.0 - @vMagScat*(0.5 - rand)
		si = Math.sin(th)*ab
		co = Math.cos(th)*ab
		x = @vel.x
		y = @vel.y
		v = cpv(x*co - y*si, x*si + y*co)
		@parts[@partIndex%@size] = AS::Particle.new(
			@pos + @scat*rand,
			v,
			@ticks-1
		)
		@partIndex += 1
	end
	
	def life(part)
		(@ticks - part.t).to_f/@life.to_f
	end
	
	def draw()
		@burst.times{emitParticle()} if(@ticks%@rate == 0)
		
		@tex.bind
		w, h = @tex.w, @tex.h
		GL.Begin(GL::QUADS)
			@parts.each do|part|
				v = @func.value(part, @ticks - part.t)
				
				l = life(part)
				li = 1.0 - l
				GL.Color(
					@colora[0]*li + @colorb[0]*l,
					@colora[1]*li + @colorb[1]*l,
					@colora[2]*li + @colorb[2]*l,
					@colora[3]*li + @colorb[3]*l
				)
				
				r = (@smax - @smin)*l + @smin
				
				GL.TexCoord(0, 0); GL.Vertex(v.x - r, v.y - r)
				GL.TexCoord(w, 0); GL.Vertex(v.x + r, v.y - r)
				GL.TexCoord(w, h); GL.Vertex(v.x + r, v.y + r)
				GL.TexCoord(0, h); GL.Vertex(v.x - r, v.y + r)
			end
		GL.End()
		
		@ticks += 1
	end
end

class LinearMotionFunc
	def value(part, t)
		part.pos + part.vel*t
	end
end

class ExpMotionFunc
	def initialize(a)
		@a = a
		@ali = 1.0/Math.log(a)
	end
	
	def value(part, t)
		v = (1.0 - @a**(-t))*@ali
		part.pos + part.vel*(v)
	end
end