PedTex = AS::Texture.open('graphics/pedestal.png', GL::NEAREST)

class Level
	attr_reader :pline, :start, :balls, :space, :name

	pw = 0.2/2
	ph = 0.5
	Pedestal = [cpv(-pw, 0.0), cpv(-pw, ph), cpv(pw, ph), cpv(pw, 0.0)]
	Scale = 128

	def initialize(pline, start, balls)
		@start = start*Scale
		@pedestals = balls.collect{|v| v*Scale - cpv(16,0)}
		
		@balls = Array.new
		balls.each do|theball|
			ball = theball*Scale
			ball.y += Scale/2.0 + Ball::R
			@balls << ball
		end
		
		@pline = PolyLine.new(pline, Scale)
		balls.each do|ball|
			@pline.add Pedestal.collect{|p| (p + ball)*Scale}
		end
		@list = @pline.to_list
		
		@space = CP::Space.new(50, 1000)
		
		@pline.each_segment do|a,b|
			@space << Segment.new(a,b)
		end
		
		@doors = Hash.new
		@buttons = Array.new
		@glass = Array.new
		@fields = Array.new
	end
	
	def add_door(name, door, isopen)
		@doors[name] = door
		close_door(name) unless isopen
	end
	
	def close_door(name)
		door = @doors[name]
		@space << door if door.open
		door.open = false
	end
	
	def open_door(name)
		door = @doors[name]
		unless door.open
			@space.remove door
			@space.pack
		end
		door.open = true
	end
	
	def add_button(button)
		@buttons << button
		@space << button
	end
	
	def add_glass(glass)
		glass.space = @space
		glass.glass = @glass
		@glass << glass
		@space << glass
	end
	
	def add_field(field)
		@fields << field
		@space << field
	end
	
	def max
		@pline.max
	end
		
	def update
		@space.update
	end
	
	def draw
		GL.Begin(GL::LINES)
			GL.Color(1,0,0)
			@doors.each_value do|door|
				door.vertexes unless door.open
			end
			
			GL.Color(0,0,1)
			@buttons.send_each(:vertexes)
			
			GL.Color(0,1,1)
			@glass.send_each(:vertexes)
			
			GL.Color(1,1,0)
			@fields.send_each(:vertexes)
		GL.End
		
		GL.Color(1,1,1)
		@pedestals.each do|ped|
			PedTex.draw_scaled(ped.x, ped.y, 2)
		end
		
#		AS.no_tex_mode		
#		@list.call_list
	end
	
	def ball_taken
		puts 'ball taken'
	end
end

class Segment < CP::Segment
	def initialize(a, b)
		super(a, b, 1, 1)
		self.type = 0;
	end
	
	def vertexes
		GL.Vertex(a.x, a.y)
		GL.Vertex(b.x, b.y)
	end
end

class Door < Segment
	attr_accessor :open

	def initialize(a, b)
		super(a*Level::Scale, b*Level::Scale)
		@open = true
	end
end

class Button < Segment
	def initialize(a, b, &func)
		super(a*Level::Scale, b*Level::Scale)
		self.type = 3
		@func = func
	end
	
	def push
		@func.call
	end
end

class Glass < Segment
	attr_accessor :space, :glass

	def initialize(a, b, &func)
		super(a*Level::Scale, b*Level::Scale)
		self.type = 5
		self.u = 0.5
		@func = func
	end
	
	def smash
		@space.remove(self)
		@glass.delete(self)
		@func.call if @func
	end
end

class Field < Segment
	def initialize(a, b)
		super(a*Level::Scale, b*Level::Scale)
		self.type = 4
		self.u = 0
		self.e = 0
	end
end
	

require 'levels/level1'
require 'levels/level2'
require 'levels/level3'
require 'levels/level4'
require 'levels/level5'
require 'levels/level6'
require 'levels/level7'
require 'levels/level8'

Levels = [
	Level1,
	Level2,
	Level3,
	Level4,
	Level5,
	Level6,
	Level7,
	Level8,
]
