ScanLines = AS::Texture.open('graphics/scanlines.png')

class Playstate < AS::Gamestate
	def add_actions
		action(AS::Event.for_key(AS::Event::Key_ESCAPE)){stop(:stop)}
	end
	
	def initialize(level)
		super()
		@level = level
		@rentex = AS::Texture.blank(512,384, GL::NEAREST)
		@tex = AS::Texture.open(@level.image, GL::NEAREST)
						
		@ship = Ship.new
		@ship.p = @level.start
		@level.space << @ship
		
		@balls = Array.new
		@level.balls.each do|pos|
			ball = Ball.new
			ball.p = pos
			@level.space << ball
			@balls << ball
		end
		
		@max = @level.max - cpv(512, 384)
		
		@attract = nil
		@grabwait = false
		@untaken = true
		
		@warp = Warp.new(@level.start)
		
		@timer = TickTimer.new(60)
		
		add_actions
	end
	
	def attract_ball
		if AS::Event.key_value(AS::Event::Key_SPACE)
#			if $easy
#				imp = @ship.v.normalize*2000.0
#			else
#				imp = @ship.rot*2000.0
#			end
#			@ship.v -= imp*@ship.m
#			@attract.v += imp*@attract.m
			v = @attract.v + @ship.v*2
			@ship.v = cpv(0,0)
			
			v = v.normalize*800 if v.length > 800
			@attract.v = v
			
			@attract.f = cpv(0,0)
			@ship.f = cpv(0,0)
			@balls << @attract
			@attract = nil
			@grabwait = 0
			ShootSound.play
		else
			delta =  @ship.p - @attract.p
			f = delta*(100000/(delta.dot(delta) + 10000))
			@attract.f = f
			@ship.f += -f
		end
	end
	
	def drive_ship
		@ship.w *= 0.9
		
		fwd = AS::Event.key_value(AS::Event::Key_UP)
		bck = AS::Event.key_value(AS::Event::Key_DOWN)
		@ship.f = @ship.rot*4000 if fwd and not bck
		@ship.f = @ship.rot*-3000 if bck and not fwd
		@ship.f = cpv(0,0) if !(fwd^bck)
		
		left = AS::Event.key_value(AS::Event::Key_LEFT)
		right = AS::Event.key_value(AS::Event::Key_RIGHT)
		@ship.t = 100000 if left and not right
		@ship.t = -100000 if right and not left
		@ship.t = 0 if !(left^right)
	end
	
	def drive_ship2
		up = AS::Event.key_value(AS::Event::Key_UP)
		down = AS::Event.key_value(AS::Event::Key_DOWN)
		vert = 4000 if up and not down
		vert = -4000 if down and not up
		vert = 0 if !(down^up)		
		
		left = AS::Event.key_value(AS::Event::Key_LEFT)
		right = AS::Event.key_value(AS::Event::Key_RIGHT)
		horiz = 4000 if right and not left
		horiz = -4000 if left and not right
		horiz = 0 if !(left^right)
		
		@ship.f = cpv(horiz, vert)	
	end
		
	def warp_ball(ball)
		if ball.p.within_dist?(@level.start, 100)
			@level.space.remove(ball)
			BallAwaySound.play
			return true
		end
		return false
	end
	
	def update
		if $easy
			drive_ship2
		else
			drive_ship
		end
		
		attract_ball if @attract

		@balls.each do|ball|		
			if not @attract and @ship.p.within_dist?(ball.p, 100) and not @grabwait
				@attract = ball
				@level.ball_taken if @untaken
				@balls.delete(ball)
				@untaken = false
				PickUpSound.play
				break
			end
		end
		
		if @grabwait
			@grabwait += 1
			@grabwait=false if @grabwait==90
		end
		
		if @attract and warp_ball(@attract)
			@level.space.remove(@attract)
			@attract = nil
		end
		
		@balls.each do|ball|
			@balls.delete(ball) if warp_ball(ball)
		end
		
		if @balls.empty? and not @attract
			stop(TimeScore.new(@timer.to_seconds))
		end
		
		@level.update
		GC.start if @timer.getTicks%60==0
	end
	
	def time_string
		sprintf("%02d:%05.2f", *@timer.to_seconds.divmod(60.0))
	end
	
	def draw
		Canvas.x = [[@ship.p.x.to_i/2*2, 512].max, @max.x].min
		Canvas.y = [[@ship.p.y.to_i/2*2, 384].max, @max.y].min
		
		Canvas.set
		
		GL.Color(1,1,1)
		AS.simple_mode
		@tex.draw_scaled(0,0,2)	
			
		AS.alpha_blend_mode
		@warp.draw
		
		GL.Color(1,1,1)
		@balls.send_each(:draw)
		if @attract
			GL.Color(1,0,0)
			@attract.draw
		end
		
		GL.Color(1,1,1)
		@ship.draw
		
		AS.no_tex_mode
		@level.draw
		
		HudCanvas.set
		GL.Color(1,1,1)
		
		AS.mult_mode
		ScanLines.draw
		
		AS.alpha_blend_mode
		str = @ship.shield_percent + "\n   Time " + time_string
		Font.draw(100,100,0.5, str)
		@timer.stepTick{}
	end
	
	def pause
		@tex.release
	end
end
